#!/usr/bin/python
# coding: UTF-8

# import
import os,sys,re,requests,time
from urllib.parse import urlparse
from bs4 import BeautifulSoup as bsp

#改行区切りでURL一覧を記載したテキストファイルのパスを渡して、URLの一覧をリストで返す
def read_urls(ulist_path):
    with open(ulist_path) as mlist:
        ulist = mlist.read().split("\n")
        mlist.close()
    return ulist

#URLで指定したページ内の画像のURLリストを返す,extは指定画像拡張子,return[0]はhostname,return[1]はpathnameで返す
def img_url_scrape(url, timeout = 10):
    response = requests.get(url, allow_redirects=False, timeout=timeout)
    if response.status_code != 200: #もしstatus_codeが200ではない、エラーを返す。
        error = Exception("HTTP status Error.\nHTTP status: " + str(response.status_code))
        raise error
    image_urls = [urlparse(url).hostname,[]] #returnの値image_urlsの定義
    image_exts = [".jpg", ".jpeg", ".png", ".gif", ".bmp"]
    soup = bsp(response.content,"html5lib")#lxmlが環境の関係でどうしてもエラーが出るので、parserにはhtml5libを使用,html5libがない場合は変更必須
    links = soup.find_all("img")
    for link in links:
        for image_ext in image_exts:
            if link.get("src").endswith(image_ext):
                image_urls[1].append(link.get("src"))
    return image_urls

#URLの画像をダウンロードする
def download_img(url, timeout = 10):
    response = requests.get(url, timeout=timeout)
    if response.status_code != 200:#もしstatus_codeが200ではないなら、エラーを返す。
        error = Exception("HTTP status Error.\nHTTP status: " + str(response.status_code))
        raise error
    content_type = response.headers["content-type"]
    if 'image' not in content_type:#もしcontent-typeがimageではないなら、エラーを起こす。
        error = Exception("Content-Type Error.\nContent-Type: " + str(content_type))
        raise error
    return response.content

# 画像を保存する
def save_img(fullpath, image):
    with open(fullpath, "wb") as file:
        file.write(image)
        file.close()

#保存フォルダパスとURLから画像の保存ファイルパスを決める
def mk_filename(base_dir, url):
    list = url.split("/")#URLを/でsplitしてlistに代入
    filename = list[len(list)-1]#URLの末尾/以下を取得、listの末尾を取得
    fullpath = os.path.join(base_dir, filename)#画像のフルパスを生成
    return fullpath


if __name__ == "__main__":
    ulist_path1 = "urls1.txt"#改行区切りでURL一覧を記載したテキストファイル指定
    current_urls = read_urls(ulist_path1) #解析対象のURLリスト
    current_imgs = []#画像URLリスト
    for link in current_urls:
        if link != "":#urlが空ではない時
            current_imgs.append(img_url_scrape(link))#

    for img_urls in current_imgs:
        hostname = img_urls[0]
        dir = os.path.join("data",hostname)#画像の保存パス、ホストネームのフォルダパスの指定(/data/hostname,dataフォルダ以下にホストネームフォルダを生成するようにした)
        print(dir)#現在の保存パスを表示
        os.makedirs(dir, exist_ok=True)
        for link in img_urls[1]:
            print("downloading..."+str(link))
            time.sleep(1)#1秒処理を止める
            if re.match("http",link):#画像パスが絶対パスの場合
                save_img(mk_filename(dir, link), download_img(link))
            elif re.match("//",link):#画像パスが//で始まる絶対パスの場合
                save_img(mk_filename(dir, link), download_img("http:" + link))
            else:#画像パスが相対パスの場合
                save_img(mk_filename(dir, link), download_img("http://" + hostname + link))
        print(hostname + " images Downloaded")


